// -*-Mode: C++;-*-

// TTY-Grin
// Copyright (C) 2001-2003 Daniel Beer
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef _MESSAGE_H
#define _MESSAGE_H

#include <string>
#include <list>
#include <vector>
#include <map>
#include <iostream>
#include <ctype.h>

class message_part {
public:
  struct ltcase_string {
    bool operator()(const string& a, const string& b) const {
      return strcasecmp(a.c_str(), b.c_str())<0;
    }
  };

  typedef list<string> receipt_list;
  typedef map<string, string, ltcase_string> header_list;
  typedef list<message_part> child_list;
  typedef vector<char> content_t;

  message_part(void);
  message_part(const message_part&);
  ~message_part(void);
  message_part& operator=(const message_part&);

  message_part(istream&, const char * =0, list<string> * =0);
  void send(ostream& out, const char * ="\r\n") const;

  receipt_list& receipts(void) { return _receipts; }
  const receipt_list& receipts(void) const { return _receipts; }

  header_list& headers(void) { return _headers; }
  const header_list& headers(void) const { return _headers; }

  child_list& children(void) { return _children; }
  const child_list& children(void) const { return _children; }

  content_t& content(void) { return _data; }
  const content_t& content(void) const { return _data; }

private:
  receipt_list _receipts;
  header_list _headers;
  child_list _children;
  content_t _data;

  void chomp(char *buf) {
    char *x=buf+strlen(buf)-1;
    while(x>=buf&&(*x=='\r'||isspace(*x))) x--;
    *(++x)=0;
  }
  int hexdigit(char x) { return isdigit(x)?x-'0':toupper(x)-'A'+10; }
  char to64(char);
  void message_part_two(istream&, const char *, list<string> *);
};

class message {
public:
  typedef list<string> raw_t;

  message(void) { }

  message(istream& in) : _raw(), _root(in, 0, &_raw) { }
  void send(ostream& out, const char *lend) const { _root.send(out, lend); }
  void send_original(ostream& out, const char *lend) const;

  message_part& root(void) { return _root; }
  const message_part& root(void) const { return _root; }

  raw_t& raw(void) { return _raw; }
  const raw_t& raw(void) const { return _raw; }

  void no_raw(void) { _raw.clear(); }

private:
  raw_t _raw;
  message_part _root;
};

#endif
