// TTY-Grin
// Copyright (C) 2001-2003 Daniel Beer
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <signal.h>
#include <ctype.h>
#include "grin.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace std;

void
tty_grin_help_draw(const tty_grin_help_t& help) {
  SLsmg_set_color(SLC_MAIN);
  SLsmg_cls();

  SLsmg_gotorc(0, 0);
  SLsmg_write_string((char *)gettext(help.title));

  unsigned int begin=0, end, line=2;

  const char *comment=gettext(help.comment);

  while(comment[begin]) {
    for(end=begin;end-begin<(unsigned)SLtt_Screen_Cols-1&&
	  comment[end]&&comment[end]!='\n';end++);
    if(end-begin>=(unsigned)SLtt_Screen_Cols-1) {
      while(end>=begin&&!isspace(comment[end])) end--;
      if(end<begin) end=begin+SLtt_Screen_Cols;
      else end++;
    }
    
    SLsmg_gotorc(line++, 0);
    SLsmg_write_nstring((char *)comment+begin, end-begin);

    begin=end;
    if(comment[begin]=='\n') begin++;
    else while(isspace(comment[begin])&&comment[begin]!='\n') begin++;
  }

  unsigned int keysize=0, commentsize=0;

  for(int i=0;help.keys[i].domain;i++) {
    string kname=kbd_reverse_translate(help.keys[i].domain,
				       help.keys[i].command);
    if(kname.size()) {
      begin=strlen(gettext(kname.c_str()));
      if(begin>keysize) keysize=begin;
      begin=strlen(gettext(help.keys[i].comment));
      if(begin>commentsize) commentsize=begin;
    }
  }

  unsigned int size=keysize+commentsize+4, kpr=SLtt_Screen_Cols/size;

  if(!kpr) kpr=1;

  for(int i=0;help.keys[i].domain;i++) {
    string kname=kbd_reverse_translate(help.keys[i].domain,
				       help.keys[i].command);
    if(kname.size()) {
      SLsmg_set_color(SLC_SELECTION);
      SLsmg_gotorc(i/kpr+line+1, (i%kpr)*size);
      SLsmg_write_string((char *)gettext(kname.c_str()));
      SLsmg_set_color(SLC_MAIN);
      SLsmg_gotorc(i/kpr+line+1, (i%kpr)*size+keysize+1);
      SLsmg_write_string((char *)gettext(help.keys[i].comment));
    }
  }
}

void
tty_grin_help_winch(int signal) {
  SLtt_get_screen_size();
  SLsmg_reinit_smg();
  tty_grin_help_draw(*(const tty_grin_help_t *)sigdata_winch);
  SLsmg_refresh();
}

void
tty_grin_help(const tty_grin_help_t& help) {
  void *sigdata_old=sigdata_winch;
  SLSig_Fun_Type *f=SLsignal(SIGWINCH, tty_grin_help_winch);
  sigdata_winch=(void *)&help;

  tty_grin_help_draw(help);
  SLsmg_refresh();

  const char *cmd=kbd_translate("help", SLkp_getkey());

  while(cmd&&!strcasecmp(cmd, "redraw")) {
    SLsmg_touch_lines(0, SLtt_Screen_Rows);
    tty_grin_help_draw(help);
    SLsmg_refresh();
    cmd=kbd_translate("help", SLkp_getkey());
  }

  SLsignal(SIGWINCH, f);
  sigdata_winch=sigdata_old;
}
